@@ -20,43 +20,25 @@
  * Solaris.
  */
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-#else
-#  ifdef HAVE_SYS_TIME_H
-#    include <sys/time.h>
-#  else
-#    include <time.h>
-#  endif
-#endif
-
 /* Make sure we define 64 bit types */
 #ifdef _WIN32
-#  define PATH_SEPARATOR "\\"
-#  define size_t unsigned int
-#  define ssize_t int
 #  define int64_t __int64
 #  define uint64_t unsigned __int64
 #  define uint32_t unsigned int
-#  define PRIu64  "I64u"
-#  define PRId64  "I64d"
 #else
-#  define PATH_SEPARATOR "/"
-#  if defined(HAVE_INTTYPES_H)
-#    include <inttypes.h>
-#  elif defined(HAVE_STDINT_H)
+#  if defined(HAVE_STDINT_H)
 #    include <stdint.h>
+#  elif defined(HAVE_INTTYPES_H)
+#    include <inttypes.h>
 #  endif
 #endif
 
-/* some defaults if not provided above */
-#ifndef SCNdMAX
-#  define SCNdMAX "lld"
+#ifdef _WIN32
+#define FORMAT_INT64      "%I64d"
+#define FORMAT_UINT64     "%I64u"
+#else
+#define FORMAT_INT64      "%lld"
+#define FORMAT_UINT64     "%llu"
 #endif
 
 #endif /* __COMPAT_H__ */
