@@ -26,9 +26,7 @@
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <time.h>
 
 #include "connection.h"
 #include "refbuf.h"
@@ -40,7 +38,6 @@
 
 #include "format_ogg.h"
 #include "format_mp3.h"
-#include "format_ebml.h"
 
 #include "logging.h"
 #include "stats.h"
@@ -55,31 +52,15 @@
 static int format_prepare_headers (source_t *source, client_t *client);
 
 
-format_type_t format_get_type (const char *contenttype)
+format_type_t format_get_type(char *contenttype)
 {
     if(strcmp(contenttype, "application/x-ogg") == 0)
         return FORMAT_TYPE_OGG; /* Backwards compatibility */
     else if(strcmp(contenttype, "application/ogg") == 0)
         return FORMAT_TYPE_OGG; /* Now blessed by IANA */
-    else if(strcmp(contenttype, "audio/ogg") == 0)
-        return FORMAT_TYPE_OGG;
-    else if(strcmp(contenttype, "video/ogg") == 0)
-        return FORMAT_TYPE_OGG;
-    else if(strcmp(contenttype, "audio/webm") == 0)
-        return FORMAT_TYPE_EBML;
-    else if(strcmp(contenttype, "video/webm") == 0)
-        return FORMAT_TYPE_EBML;
-    else if(strcmp(contenttype, "audio/x-matroska") == 0)
-        return FORMAT_TYPE_EBML;
-    else if(strcmp(contenttype, "video/x-matroska") == 0)
-        return FORMAT_TYPE_EBML;
-    else if(strcmp(contenttype, "video/x-matroska-3d") == 0)
-        return FORMAT_TYPE_EBML;
     else
         /* We default to the Generic format handler, which
-           can handle many more formats than just mp3.
-	   Let's warn that this is not well supported */
-	ICECAST_LOG_WARN("Unsupported or legacy stream type: \"%s\". Falling back to generic minimal handler for best effort.", contenttype);
+           can handle many more formats than just mp3 */
         return FORMAT_TYPE_GENERIC;
 }
 
@@ -91,9 +72,6 @@ int format_get_plugin(format_type_t type, source_t *source)
     case FORMAT_TYPE_OGG:
         ret = format_ogg_get_plugin (source);
         break;
-    case FORMAT_TYPE_EBML:
-        ret = format_ebml_get_plugin (source);
-        break;
     case FORMAT_TYPE_GENERIC:
         ret = format_mp3_get_plugin (source);
         break;
@@ -115,17 +93,15 @@ static void find_client_start (source_t *source, client_t *client)
 {
     refbuf_t *refbuf = source->burst_point;
 
-    /* we only want to attempt a burst at connection time, not midstream
-     * however streams like theora may not have the most recent page marked as
-     * a starting point, so look for one from the burst point */
-    if (client->intro_offset == -1 && source->stream_data_tail
-            && source->stream_data_tail->sync_point)
+    /* we only want to attempt a burst at connection time, not midstream */
+    if (client->intro_offset == -1)
         refbuf = source->stream_data_tail;
     else
     {
-        size_t size = client->intro_offset;
+        long size = 0;
         refbuf = source->burst_point;
-        while (size > 0 && refbuf && refbuf->next)
+        size = client->intro_offset;
+        while (size > 0 && refbuf->next)
         {
             size -= refbuf->len;
             refbuf = refbuf->next;
@@ -150,7 +126,7 @@ static void find_client_start (source_t *source, client_t *client)
 static int get_file_data (FILE *intro, client_t *client)
 {
     refbuf_t *refbuf = client->refbuf;
-    size_t bytes;
+    int bytes;
 
     if (intro == NULL || fseek (intro, client->intro_offset, SEEK_SET) < 0)
         return 0;
@@ -158,7 +134,7 @@ static int get_file_data (FILE *intro, client_t *client)
     if (bytes == 0)
         return 0;
 
-    refbuf->len = (unsigned int)bytes;
+    refbuf->len = bytes;
     return 1;
 }
 
@@ -221,18 +197,15 @@ int format_check_http_buffer (source_t *source, client_t *client)
 
     if (client->respcode == 0)
     {
-        ICECAST_LOG_DEBUG("processing pending client headers");
+        DEBUG0("processing pending client headers");
 
+        client->respcode = 200;
         if (format_prepare_headers (source, client) < 0)
         {
-            ICECAST_LOG_ERROR("internal problem, dropping client");
+            ERROR0 ("internal problem, dropping client");
             client->con->error = 1;
             return -1;
         }
-        client->respcode = 200;
-        stats_event_inc (NULL, "listeners");
-        stats_event_inc (NULL, "listener_connections");
-        stats_event_inc (source->mount, "listener_connections");
     }
 
     if (client->pos == refbuf->len)
@@ -299,29 +272,8 @@ static int format_prepare_headers (source_t *source, client_t *client)
     ptr = client->refbuf->data;
     client->respcode = 200;
 
-    bytes = util_http_build_header(ptr, remaining, 0, 0, 200, NULL, source->format->contenttype, NULL, NULL, source);
-    if (bytes <= 0) {
-        ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
-        client_send_500(client, "Header generation failed.");
-        return -1;
-    } else if ((bytes + 1024) >= remaining) { /* we don't know yet how much to follow but want at least 1kB free space */
-        void *new_ptr = realloc(ptr, bytes + 1024);
-        if (new_ptr) {
-            ICECAST_LOG_DEBUG("Client buffer reallocation succeeded.");
-            client->refbuf->data = ptr = new_ptr;
-            client->refbuf->len = remaining = bytes + 1024;
-            bytes = util_http_build_header(ptr, remaining, 0, 0, 200, NULL, source->format->contenttype, NULL, NULL, source);
-            if (bytes <= 0 || bytes >= remaining) {
-                ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
-                client_send_500(client, "Header generation failed.");
-                return -1;
-            }
-        } else {
-            ICECAST_LOG_ERROR("Client buffer reallocation failed. Dropping client.");
-            client_send_500(client, "Buffer reallocation failed.");
-            return -1;
-        }
-    }
+    bytes = snprintf (ptr, remaining, "HTTP/1.0 200 OK\r\n"
+            "Content-Type: %s\r\n", source->format->contenttype);
 
     remaining -= bytes;
     ptr += bytes;
@@ -357,22 +309,7 @@ static int format_prepare_headers (source_t *source, client_t *client)
             if (strcasecmp(var->name, "ice-password") &&
                 strcasecmp(var->name, "icy-metaint"))
             {
-		if (!strcasecmp(var->name, "ice-name"))
-		{
-		    ice_config_t *config;
-		    mount_proxy *mountinfo;
-
-		    config = config_get_config();
-		    mountinfo = config_find_mount (config, source->mount, MOUNT_TYPE_NORMAL);
-
-		    if (mountinfo && mountinfo->stream_name)
-		        bytes = snprintf (ptr, remaining, "icy-name:%s\r\n", mountinfo->stream_name);
-                    else
-		        bytes = snprintf (ptr, remaining, "icy-name:%s\r\n", var->value);
-
-                    config_release_config();
-		}
-                else if (!strncasecmp("ice-", var->name, 4))
+                if (!strncasecmp("ice-", var->name, 4))
                 {
                     if (!strcasecmp("ice-public", var->name))
                         bytes = snprintf (ptr, remaining, "icy-pub:%s\r\n", var->value);
@@ -392,13 +329,6 @@ static int format_prepare_headers (source_t *source, client_t *client)
             }
         }
 
-        if (bytes < 0 || bytes >= remaining) {
-            avl_tree_unlock(source->parser->vars);
-            ICECAST_LOG_ERROR("Can not allocate headers for client %p", client);
-            client_send_500(client, "Header generation failed.");
-            return -1;
-        }
-
         remaining -= bytes;
         ptr += bytes;
         if (next)
@@ -406,23 +336,18 @@ static int format_prepare_headers (source_t *source, client_t *client)
     }
     avl_tree_unlock(source->parser->vars);
 
+    bytes = snprintf (ptr, remaining, "Server: %s\r\n", ICECAST_VERSION_STRING);
+    remaining -= bytes;
+    ptr += bytes;
+
     bytes = snprintf (ptr, remaining, "\r\n");
-    if (bytes < 0 || bytes >= remaining) {
-        ICECAST_LOG_ERROR("Can not allocate headers for client %p", client);
-        client_send_500(client, "Header generation failed.");
-        return -1;
-    }
     remaining -= bytes;
     ptr += bytes;
 
     client->refbuf->len -= remaining;
     if (source->format->create_client_data)
-        if (source->format->create_client_data (source, client) < 0) {
-            ICECAST_LOG_ERROR("Client format header generation failed. "
-                "(Likely not enough or wrong source data) Dropping client.");
-            client->respcode = 500;
+        if (source->format->create_client_data (source, client) < 0)
             return -1;
-        }
     return 0;
 }
 
